@@ -2,7 +2,7 @@
 /*                                                                        */
 /*  This file is part of Frama-C.                                         */
 /*                                                                        */
-/*  Copyright (C) 2007-2018                                               */
+/*  Copyright (C) 2007-2019                                               */
 /*    CEA (Commissariat à l'énergie atomique et aux énergies              */
 /*         alternatives)                                                  */
 /*                                                                        */
@@ -25,7 +25,9 @@
 #include "features.h"
 __PUSH_FC_STDLIB
 
+#include "__fc_builtin.h"
 #include "__fc_define_uid_and_gid.h"
+#include "__fc_string_axiomatic.h"
 
 // for size_t
 #include "stddef.h"
@@ -37,12 +39,44 @@ struct passwd {
   char    *pw_passwd; // not POSIX, but allowed by it, and present in glibc
   uid_t    pw_uid;
   gid_t    pw_gid;
+  char    *pw_gecos; // not POSIX, but present in most implementations
   char    *pw_dir;
   char    *pw_shell;
 };
 
-extern struct passwd *getpwnam(const char *);
-extern struct passwd *getpwuid(uid_t);
+extern char __fc_getpwuid_pw_name[64];
+extern char __fc_getpwuid_pw_passwd[64];
+extern char __fc_getpwuid_pw_dir[64];
+extern char __fc_getpwuid_pw_shell[64];
+
+struct passwd __fc_pwd =
+  {.pw_name = __fc_getpwuid_pw_name,
+   .pw_passwd = __fc_getpwuid_pw_passwd,
+   .pw_dir = __fc_getpwuid_pw_dir,
+   .pw_shell = __fc_getpwuid_pw_shell};
+
+struct passwd *__fc_p_pwd = & __fc_pwd;
+
+/*@
+  // missing: may assign to errno: EIO, EINTR, EMFILE, ENFILE
+  // missing: assigns \result, __fc_pwd[0..] \from 'password database'
+  requires valid_name: valid_read_string(name);
+  assigns \result \from __fc_p_pwd, indirect:name[0..];
+  assigns __fc_pwd \from indirect:name[0..];
+  ensures result_null_or_internal_struct:
+    \result == \null || \result == __fc_p_pwd;
+*/
+extern struct passwd *getpwnam(const char *name);
+
+/*@ // missing: assigns \result, __fc_pwd[0..] \from 'password database'
+  assigns \result \from __fc_p_pwd, indirect:uid;
+  assigns __fc_pwd \from indirect:uid;
+  ensures result_null_or_internal_struct:
+    \result == \null || \result == __fc_p_pwd;
+*/
+extern struct passwd *getpwuid(uid_t uid);
+
+
 extern int            getpwnam_r(const char *, struct passwd *, char *,
 			  size_t, struct passwd **);
 extern int            getpwuid_r(uid_t, struct passwd *, char *,
