@@ -2,7 +2,7 @@
 /*                                                                        */
 /*  This file is part of Frama-C.                                         */
 /*                                                                        */
-/*  Copyright (C) 2007-2018                                               */
+/*  Copyright (C) 2007-2019                                               */
 /*    CEA (Commissariat à l'énergie atomique et aux énergies              */
 /*         alternatives)                                                  */
 /*                                                                        */
@@ -222,7 +222,6 @@ extern char *strpbrk(const char *s, const char *accept);
   @*/
 extern char *strstr(const char *haystack, const char *needle);
 
-#ifdef __USE_GNU
 /*@ requires valid_string_haystack: valid_read_string(haystack);
   @ requires valid_string_needle: valid_read_string(needle);
   @ assigns \result \from haystack, indirect:haystack[0..],
@@ -232,7 +231,6 @@ extern char *strstr(const char *haystack, const char *needle);
   @   || (\subset(\result, haystack+(0..)) && \valid_read(\result));
   @*/
 extern char *strcasestr (const char *haystack, const char *needle);
-#endif
 
 // internal state of strtok
 char *__fc_strtok_ptr;
@@ -374,9 +372,19 @@ extern char *strcpy(char *restrict dest, const char *restrict src);
 extern char *strncpy(char *restrict dest,
 		     const char *restrict src, size_t n);
 
+/*@ // Non-POSIX, but often present
+  @ requires valid_string_src: valid_read_string(src);
+  @ requires room_nstring: \valid(dest+(0..n-1));
+  @ requires separation:
+  @   \separated(dest+(0..n-1), src+(0..\max(n-1,strlen(src))));
+  @ assigns dest[0..n-1] \from src[0..n-1];
+  @ assigns \result \from indirect:src, indirect:src[0..n-1], indirect:n;
+  @ ensures initialization: \initialized(dest+(0..\min(strlen(src),n-1)));
+  @ ensures bounded_result: \result == strlen(src);
+ */
+size_t strlcpy(char * restrict dest, const char * restrict src, size_t n);
+
 // stpcpy is POSIX.1-2008
-#ifdef _POSIX_C_SOURCE
-# if _POSIX_C_SOURCE >= 200809L
 /*@ requires valid_string_src: valid_read_string(src);
   @ requires room_string: \valid(dest+(0..strlen(src)));
   @ requires separation:
@@ -387,8 +395,6 @@ extern char *strncpy(char *restrict dest,
   @ ensures points_to_end: \result == dest + strlen(dest);
   @*/
 extern char *stpcpy(char *restrict dest, const char *restrict src);
-# endif
-#endif
 
 /*@ // missing: separation
   @ requires valid_string_src: valid_read_string(src);
@@ -429,6 +435,17 @@ extern char *strcat(char *restrict dest, const char *restrict src);
   @*/
 extern char *strncat(char *restrict dest, const char *restrict src, size_t n);
 
+/*@ // Non-POSIX, but often present
+  @ // missing: separation
+  @ requires valid_string_src: valid_read_string(src);
+  @ requires valid_string_dest: valid_string(dest);
+  @ requires room_nstring: \valid(dest+(0..n-1));
+  @ assigns dest[strlen(dest)..n] \from indirect:n, src[0..strlen(src)];
+  @ assigns \result \from indirect:src, indirect:src[0..n-1], indirect:n;
+  @ ensures bounded_result: \result == strlen(dest) + strlen(src);
+  @*/
+extern size_t strlcat(char *restrict dest, const char *restrict src, size_t n);
+
 /*@ // missing: separation
   @ requires valid_dest: \valid(dest+(0..n - 1));
   @ requires valid_string_src: valid_read_string(src);
@@ -479,15 +496,24 @@ extern char *strdup (const char *s);
 extern char *strndup (const char *s, size_t n);
 
 // More POSIX, non-C99 functions
-#ifdef _POSIX_C_SOURCE
 extern char *stpncpy(char *restrict dest, const char *restrict src, size_t n);
 //extern int strcoll_l(const char *s1, const char *s2, locale_t locale);
 //extern char *strerror_l(int errnum, locale_t locale);
 extern int strerror_r(int errnum, char *strerrbuf, size_t buflen);
-extern char *strsignal(int sig);
+
+extern char __fc_strsignal[64];
+char * const __fc_p_strsignal = __fc_strsignal;
+
+/*@ //missing: requires valid_signal(signum);
+  @ assigns \result \from __fc_p_strsignal, indirect:signum;
+  @ ensures result_internal_str: \result == __fc_p_strsignal;
+  @ ensures result_nul_terminated: \result[63] == 0;
+  @ ensures result_valid_string: valid_read_string(\result);
+  @*/
+extern char *strsignal(int signum);
+
 //extern size_t strxfrm_l(char *restrict s1, const char *restrict s2, size_t n,
 //                        locale_t locale);
-#endif
 
 __END_DECLS
 
