@@ -2,7 +2,7 @@
 /*                                                                        */
 /*  This file is part of Frama-C.                                         */
 /*                                                                        */
-/*  Copyright (C) 2007-2018                                               */
+/*  Copyright (C) 2007-2019                                               */
 /*    CEA (Commissariat à l'énergie atomique et aux énergies              */
 /*         alternatives)                                                  */
 /*                                                                        */
@@ -31,7 +31,7 @@ __PUSH_FC_STDLIB
 #include "__fc_string_axiomatic.h"
 
 #include "errno.h"
-
+#include "signal.h"
 /*
  * Names of the interval timers, and structure
  * defining a timer setting:
@@ -104,8 +104,19 @@ extern time_t mktime(struct tm *timeptr);
 */
 extern time_t time(time_t *timer);
 
+char __fc_ctime[26];
+char * const  __fc_p_ctime = __fc_ctime;
+
 extern char *asctime(const struct tm *timeptr);
 
+/*@
+  requires valid_timer: \valid_read(timer);
+  requires initialization:init_timer: \initialized(timer);
+  assigns __fc_ctime[0..25] \from indirect:*timer, indirect:__fc_time;
+  assigns \result \from indirect:*timer, indirect:__fc_time, __fc_p_ctime;
+  ensures result_points_to_ctime: \result == __fc_p_ctime;
+  ensures result_valid_string: valid_read_string(__fc_p_ctime);
+*/
 extern char *ctime(const time_t *timer);
 
 struct tm __fc_time_tm;
@@ -158,7 +169,7 @@ extern int clock_getres(clockid_t, struct timespec *);
 #else
     // simulates a system without monotonic clock
     assigns \result\from clk_id;
-    ensures error: \result == EINVAL
+    ensures error: \result == EINVAL;
 #endif
   behavior bad_clock_id:
     assumes bad_id: clk_id != CLOCK_REALTIME && clk_id != CLOCK_MONOTONIC;
